"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getConfig = void 0;
const path_1 = __importDefault(require("path"));
const utils_1 = require("@umijs/utils");
const api_extractor_1 = require("@microsoft/api-extractor");
const DEFAULT_OUTPUT_DIR = './compiled';
/**
 * return type field value for pkg
 */
function getTypeFieldForPkg(pkg) {
    return pkg.type || pkg.types || pkg.typing || pkg.typings;
}
/**
 * get type info for pkg or @types/pkg
 */
function getTypeInfoForPkg(pkgPath) {
    const pkg = require(pkgPath);
    const info = { pkgPath: pkgPath, dtsPath: getTypeFieldForPkg(pkg) };
    if (info.dtsPath) {
        // resolve builtin types
        info.dtsPath = path_1.default.resolve(path_1.default.dirname(pkgPath), info.dtsPath);
    }
    else {
        // resolve @types/xxx pkg
        try {
            info.pkgPath = require.resolve(`@types/${pkg.name.replace('@', '').replace('/', '__')}/package.json`, { paths: [pkgPath] });
            info.dtsPath = path_1.default.resolve(path_1.default.dirname(info.pkgPath), getTypeFieldForPkg(require(info.pkgPath)));
        }
        catch {
            return null;
        }
    }
    return info;
}
function getDtsConfig(opts) {
    return {
        pkg: require(opts.pkgPath),
        output: opts.outputPath,
        // generate prepare config for @microsoft/api-extractor
        _maePrepareConfig: {
            configObject: {
                mainEntryPointFilePath: opts.dtsPath,
                projectFolder: path_1.default.dirname(opts.pkgPath),
                // enable dts rollup feature
                dtsRollup: {
                    enabled: true,
                    publicTrimmedFilePath: opts.outputPath,
                },
                // configure ts compiler
                compiler: {
                    overrideTsconfig: {
                        compilerOptions: {
                            target: 'es5',
                            module: 'commonjs',
                            moduleResolution: 'node',
                            strict: true,
                            skipLibCheck: true,
                        },
                        include: [opts.cwd],
                    },
                },
                // configure logger
                messages: {
                    extractorMessageReporting: {
                        // only log error by default
                        default: { logLevel: 'error' },
                        // omit release tag checking
                        'ae-missing-release-tag': { logLevel: 'none' },
                    },
                },
            },
            configObjectFullPath: undefined,
            packageJsonFullPath: opts.pkgPath,
        },
        // after externals ready, generate final extractor config below
        maeConfig: undefined,
    };
}
function getConfig(opts) {
    // external project dependencies by default
    const pkgExternals = Object.keys(Object.assign({}, opts.pkg.dependencies, opts.pkg.peerDependencies)).reduce((r, dep) => ({ ...r, [dep]: dep }), {});
    const depExternals = {};
    const config = { deps: {}, dts: {} };
    const { deps, extraExternals = {}, extraDtsDeps = [] } = opts.userConfig;
    // process deps config
    Object.entries(deps).forEach(([dep, depConfig]) => {
        var _a;
        // handle array deps
        const depName = Array.isArray(deps) ? deps[parseInt(dep)] : dep;
        depConfig = Array.isArray(deps) ? {} : depConfig;
        const depEntryPath = require.resolve(depName, { paths: [opts.cwd] });
        const depPkgPath = require.resolve(`${depName}/package.json`, {
            paths: [opts.cwd],
        });
        const depTypeInfo = depConfig.dts !== false ? getTypeInfoForPkg(depPkgPath) : null;
        const depPkg = require(depPkgPath);
        // generate bundle config
        config.deps[depEntryPath] = {
            nccConfig: {
                minify: (_a = depConfig.minify) !== null && _a !== void 0 ? _a : true,
                target: 'es5',
                quiet: true,
                externals: {},
            },
            pkg: depPkg,
            output: path_1.default.resolve(opts.cwd, depConfig.output || `${DEFAULT_OUTPUT_DIR}/${depPkg.name}/index.js`),
        };
        // generate api-extractor config
        if (depTypeInfo) {
            const outputFilePath = config.deps[depEntryPath].output.replace('.js', '.d.ts');
            config.dts[depTypeInfo.dtsPath] = getDtsConfig({
                cwd: opts.cwd,
                pkgPath: depPkgPath,
                dtsPath: depTypeInfo.dtsPath,
                outputPath: outputFilePath,
            });
        }
        // prepare deps externals
        depExternals[depPkg.name] = config.deps[depEntryPath].output;
    });
    // process extraDtsDeps config
    extraDtsDeps.forEach((pkg) => {
        const depTypeInfo = getTypeInfoForPkg(require.resolve(`${pkg}/package.json`, { paths: [opts.cwd] }));
        if (depTypeInfo) {
            config.dts[depTypeInfo.dtsPath] = getDtsConfig({
                cwd: opts.cwd,
                pkgPath: depTypeInfo.pkgPath,
                dtsPath: depTypeInfo.dtsPath,
                outputPath: path_1.default.resolve(opts.cwd, `${DEFAULT_OUTPUT_DIR}/${pkg}/index.d.ts`),
            });
        }
    });
    // process externals for deps
    Object.values(config.deps).forEach((depConfig) => {
        const rltDepExternals = Object.entries(depExternals).reduce((r, [dep, target]) => {
            // skip self
            if (dep !== depConfig.pkg.name) {
                // transform dep externals path to relative path
                r[dep] = (0, utils_1.winPath)(path_1.default.relative(path_1.default.dirname(depConfig.output), path_1.default.dirname(target)));
            }
            return r;
        }, {});
        depConfig.nccConfig.externals = {
            ...pkgExternals,
            ...rltDepExternals,
            ...extraExternals,
        };
    });
    // process externals for dts
    Object.values(config.dts).forEach((dtsConfig) => {
        // always skip bundle external pkgs
        dtsConfig._maePrepareConfig.configObject.bundledPackages = Object.keys(dtsConfig.pkg.dependencies || {}).filter((name) => {
            name = name.replace('@types/', '');
            return !depExternals[name] && !extraExternals[name];
        });
        // generate the final extract config
        dtsConfig.maeConfig = api_extractor_1.ExtractorConfig.prepare(dtsConfig._maePrepareConfig);
    });
    return config;
}
exports.getConfig = getConfig;
